* mer04_smplprb.do
* Chapter 4 - Measures of disease frequency
* Sample problems
	version 12
	clear all
	set more off , permanent
	cd c:\mer\smplprb\mer04		/* this line is computer dependent */
	* capture log close 
	* log using mer04_smplprb, name(mer04_smplprb) text replace 

use c:\mer\data\mi.dta, clear
	* eliminate some variables not needed of these analyses
	drop white-mar_c2 prchf-smkces

* generate variable for observation time in years with a max. of 5 yrs
* 1 day was added so all individuals were considered to have at least 1 day of observation
* because deaths on day of admission were listed as having 0 days survival
	gen obs_5yr=(surv_mi+1)/365.25
	replace obs_5yr=5 if obs_5yr>5
* generate a variable for 5 year survival
	gen died_5yr=0
	replace died_5yr=1 if died==1 & obs_5yr<5
* generate a variable if observed for full 5yr
	gen obs_full5yr=obs_5yr==5 

* computing 5-yr risk
	* first determine how many deaths and censorings
		tab obs_full5yr died_5yr
		* there were 1406 deaths, 528 censored observations and 
		* 1031 individiuals followed for the full period (not dieing)
	* (incorrectly) ignoring withdrawals
		tab died_5yr
	* assuming that 528 withdrawals occurrred at midpoint (2.5 yrs)
		display 1406/(2965-0.5*528)
	
* computing annual incidence rate
	* approximate calculation - assuming withdrawals occurred at midpoint
		display 1406/((2965 - 0.5*(1406+528))*5)
	* exact calculation
		* compute total observation time at risk for all people (mean * # observations)
			sum obs_5yr
			display 2.980713*2965
				* there were 8837.8 years of observation time
		* compute exact incidence rate
			display 1406/8837.8
	* re-doing the calculations based on a 1 year follow-up period
	preserve
		* generate variable for observation time in years with a max. of 1 yr
		* 1 day was added so all individuals were considered to have at least 1 day of observation
		* because deaths on day of admission were listed as having 0 days survival
			gen obs_1yr=(surv_mi+1)/365.25
			replace obs_1yr=1 if obs_1yr>1
		* generate a variable for 1 year survival
			gen died_1yr=0
			replace died_1yr=1 if died==1 & obs_1yr<1
		* generate a variable if observed for full yr
			gen obs_full1yr=obs_1yr==1
		tab obs_full1yr died_1yr
		* there were 777 deaths, no censored observations and 
		* 2188 individiuals followed for the full period (not dying)
		* compute total time at risk for all people (mean * # observations)
			sum obs_1yr
			display 0.7960377*2965
				* there were 2360.25 years of observation time
		* compute exact incidence rate
			display 777/2360.25
		* the incidence rate was 0.329 = 32.9 deaths per 100 people
	restore

* estimating 1-yr risk from rate - assuming closed population
	display 1-exp(-0.1591)
	* estimating 5-yr risk
	display 1-exp(-5*0.1591)
			
* using -epitab- to compute risks and rates
	cs died_5yr sex 		/* crude est. of 5-yr risk */
	ir died_5y sex obs_5yr	/* exact est. of annual rate */

* compute prevalence of obesity at time of admission
	gen obese=bmi>30 if !missing(bmi)	/* ??? check level ??? */
	tab obese
	
* SE and confidence intervals
	ci died_5yr	/* ordinary Wald-type CI for a continuous variable */
	ci died_5yr, binomial	/* exact CI fr proportion */
	ci died_5yr, poisson exposure(obs_5yr)	/* CI for a rate */
	
* direct age standardization of hospital specific risks
* generate a 4 category age variable
	egen age_c4=cut(age), at(0 60 75 85 9999) icodes
* determine if -died_5yr- varies by age group and hospital
	tab hosp age_c4 , row chi		/* substantial differences */
	tab hosp died_5yr, row chi		/* substantial differences */
* collapse the data to hospital and age class groups
	preserve
		keep if !missing(age_c4)
		collapse (sum) died_5yr (count) n=died_5yr, by(hosp age_c4)
			format died_5yr n %6.0f
	* compute direct standardized rates
		dstdize died_5yr n age_c4, by(hosp)
			* generate a list of the collapsed data
				list, sep(4)
	restore

