* mer13_smplprb.do
* Chapter 13 - Confounding
* Sample problems
	version 12
	clear all
	set more off , permanent
	cd c:\mer\smplprb\mer13	/* this line is computer dependent */
	* capture log close 
	* log using mer13_smplprb, name(mer13_smplprb) text replace 

use c:\mer\data\mi.dta, clear
* generate a variable for 1 year survival
	* first verify that there were no censored obs. at < 1 year
		codebook surv_mi if died==0
	gen died_1yr=(died==1 & surv_mi<365)
* generate a dichotomous (0/1) variable for obesity (bmi>30)
	gen obese=bmi>=30 & !missing(bmi)
* generate both a dichotomous variable for age (split at 75) and a 
	* 5 category variable (split at 60, 70, 80 and 90)
	egen age_c2=cut(age), at(0 75 999) icodes
	egen age_c5=cut(age), at(0 60 70 80 90 999) icodes
* keep only variables needed of these analyses
	keep id died_1yr age_c2 age_c5 sex white obese prmi ptca
	sum
* save the new dataset 
	save mer_13_smplprb.dta, replace

* draw a causal diagram and check correlations among variables
	use mer_13_smplprb.dta, clear
	corr sex-age_c2
	
* crude relationship between -obese- and death
	cs died_1yr obese
		* RR is 0.564 based on 2965 obs.

* control for age (age_c5) by matching
	* the following block of code randomly selects non-obese people within each ///
		* age group so that the age distn. is same for obese and non-obese individuals
		set seed 94827
		tab age_c5 obese
		capture drop ao*
		egen aogrp=group(age_c5 obese)
		bysort aogrp: gen aoN=_N
		sort age_c5 id
		by age_c5: egen aoN0=min(aoN)
		by age_c5: egen aoN1=max(aoN)
		gen ao_prp=aoN0/aoN1
			* br age_c5 obese aogrp aoN aoN0 aoN1 ao_prp
		gen aorand=uniform()
		keep if (obese==1) | (obese==0 & aorand<=ao_prp)
		* verify that age distn. approx equal in obese and non-obese groups
			tab age_c5 obese
	* evaluate effect of -obese-
		cs died_1yr obese
			* RR is 0.723 based on 1667 obs. 
	* reload the full dataset
		use mer_13_smplprb.dta, clear

* control for sex (analytically)
	cs died_1yr obese, by(sex)
		cs obese sex
		cs died_1yr sex if obese==0
* control for age 
	cs died_1yr obese, by(age_c2)
		cs obese age_c2
		cs died_1yr age_c2 if obese==0
	* now control for age as a 5 category variable
	cs died_1yr obese, by(age_c5)

* crude relationship between angioplasty (-ptca-) and death (-died_1yr-)
	cs died_1yr ptca

* control for age (-age_c2-)
	cs died_1yr ptca, by(age_c2)
		cs ptca age_c2
		cs died_1yr age_c2 if ptca==0
* control for previous MI (-prmi-)
	cs died_1yr ptca, by(prmi)
	* evaluate interaction on the additive scale
		cs died_1yr ptca if prmi==0
		cs died_1yr ptca if prmi==1

* propensity scores
	* remove all obs. with missing values for any predictors
	keep if !missing(ptca) & !missing(sex) & !missing(age_c5) & !missing(white) ///
		& !missing(obese) & !missing(prmi)
		* this is done solely because the help file for this program suggests it should be done
	* first compute unconditional association between -died_1yr- and -ptca- 
		cs died_1yr ptca
	* compute propensity scores (based on -sex- -age_c5- -white- -obese- -prmi-)
	capture drop ps
	capture drop block
	xi:pscore ptca sex i.age_c5 white obese prmi, pscore(ps) logit comsup ///
			blockid(block) numblo(20)
			* use -detail- option to find variable/block not balanced
			* only -prmi- is unbalanced and only in 1 of 16 blocks and it is only barely
			* declared unbalanced in that block (P=0.009 with default threshold set to 0.01)
			* so we will accept the "balancedness" of the ps
			codebook ps if comsup==1
			tab block, sum(ps)
	* recompute unconditional assoc. using obs. with common support
		tab comsup
			* only 2 observations were outside region of common support
		cs died_1yr ptca if comsup==1, or
		* unconditional risk difference is -0.275
		* unconditional OR is 0.202
* nearest neighbour matching restricted to area of common support
		attnd died_1yr ptca ,  comsup pscore(ps) detail
			* appears to reduce effect of -ptca- (att reduced from -0.275 to -0.150)
* using pscore in a stratified analysis
	atts died_1yr ptca, pscore(ps) blockid(block) comsup detail
	* reduces effect of -ptca- (att=-0.157) almost as much as n.n. matching
* using pscore in a logistic regression model
	* unconditional OR
		logit died_1yr ptca, or				/* OR = 0.201 */
			estimates store uncon
	* OR adjusted for ps - either continuous or categorical
		logit died_1yr ptca ps if comsup, or				/* OR = 0.310 */
			estimates store ps_cont
		logit died_1yr ptca i.block if comsup, or	/* OR = 0.311 */
			estimates store ps_cat
	* comparison with direct control of confounders
		  logit died_1yr ptca sex i.age_c5 white obese prmi if comsup, or 	/* OR = 0.303, p<0.001 */
			estimates store cond
		* without limiting to region of common support
			logit died_1yr ptca sex i.age_c5 white obese prmi , or
	estimates table uncon ps_cont ps_cat cond, eform


	

