* Chapter 14 - Linear regression
* Sample problems
* mer14_smplprb.do
	version 12
	clear all
	set more off , permanent
	cd c:\mer\smplprb\mer14		/* this line is computer dependent */
	* capture log close 
	* log using mer14_smplprb, name(mer04_smplprb) text replace 

* creating the dataset to be analysed
	use "C:\mer\data\brazil.dta", clear
		* keep only records of people who reported 1 or more days with diarrhea
		keep if di_d_ln!=.
		* compute proportion of municipality that was rural
		gen m_prp_rural= m_pop_rural/ m_pop_total
		* keep selected variables
		keep mun comm fam id di_d di_d_ln fam_n age sex cistern income_social ///
			c_rec m_prp_rural m_hdi
	save brazil_lin_reg_exc.dta, replace

********************************************
* Exercise # 1
********************************************

use brazil_lin_reg_exc.dta, clear
* (a) distribution of outcome di_d, di_d_ln
	sum di_d di_d_ln
	histogram di_d, bin(10) name(raw, replace)
	histogram di_d_ln, bin(10) name(ln, replace)

* (b) simple linear regressions
	sum fam_n age sex cistern income_social c_rec m_hdi m_prp_rural
		* sex, cistern and income_social were dichotomous, all others are continuous 
	reg di_d_ln fam_n
	reg di_d_ln age 
	reg di_d_ln sex
	reg di_d_ln cistern
	reg di_d_ln income_social
	reg di_d_ln c_rec
	reg di_d_ln m_hdi
	reg di_d_ln m_prp_rural
	codebook m_hdi
	
* (c) understanding root MSE
	sum di_d_ln
	reg di_d_ln cistern
		predict res, res
		sum res


* (d) comparing coefficients for -cistern- from 2 models
	reg di_d_ln cistern
		estimates store smpl
	reg di_d_ln fam_n age sex cistern income_social ///
		c_rec m_hdi m_prp_rural 
		estimates store mltpl
	estimates table smpl mltpl

* comparing coefficients for -m_hdi- from 2 models
	reg di_d_ln m_hdi
		estimates store smpl_hdi
	reg di_d_ln fam_n age sex cistern income_social ///
		c_rec m_hdi m_prp_rural 
		estimates store mltpl_hdi
	estimates table smpl_hdi mltpl_hdi
	corr m_hdi fam_n age sex cistern income_social c_rec m_prp_rural

* (e) predicion intervals for means and individuals (based only on -m_hdi-)
	reg di_d_ln m_hdi
	* doing the calculations yourself
		predict pv, xb
		predict pv_mean_se, stdp
			gen pv_mean_u=pv + 1.96*pv_mean_se
			gen pv_mean_l=pv - 1.96*pv_mean_se
		predict pv_ind_se, stdf
			gen pv_ind_u=pv + 1.96*pv_ind_se
			gen pv_ind_l=pv - 1.96*pv_ind_se
		twoway (scatter di_d_ln m_hdi, msize(vsmall)) ///
			(line pv m_hdi) ///
			(line pv_mean_u m_hdi) (line pv_mean_l m_hdi) ///
			(line pv_ind_u m_hdi) (line pv_ind_l m_hdi), name(pred1, replace)
	* using the bulit in graphics capabilities
		twoway (scatter di_d_ln m_hdi, sort msize(vsmall)) ///
			(lfitci di_d_ln m_hdi, ciplot(rline)) ///
			(lfitci di_d_ln m_hdi, stdf ciplot(rline)), name(pred2, replace)

* (f) R2
	reg di_d_ln cistern
	reg di_d_ln fam_n age sex cistern income_social ///
		c_rec m_prp_rural m_hdi

********************************************
* Linear regression exercise # 2
********************************************

* open the revised "Brazil" data set 
	use brazil_lin_reg_exc.dta, clear

* (a) -c_rec- as sole predictor
	reg di_d_ln c_rec			/* continuous predictor */
		estimates store cont
	egen c_rec_c4=cut(c_rec), at(0 17 32 47 999) icodes
	reg di_d_ln i.c_rec_c4	/* categorical predictor */
		estimates store categ
	estimates table cont categ

* (b) improving interpetability of intercept
	reg di_d_ln fam_n age sex cistern income_social ///
		c_rec m_prp_rural m_hdi
	gen fam_n_min=fam_n-2
	gen age_ct=age-16.6
	gen c_rec_50=c_rec-50
	gen m_hdi_ct=m_hdi-.608
	gen m_prp_rural_25=m_prp_rural-.25
	reg di_d_ln fam_n_min age_ct sex cistern income_social ///
		c_rec_50 m_prp_rural_25 m_hdi_ct

* (c) linearity of -age- vs -di_d_ln- relationship
	twoway (scatter di_d_ln age)(lowess di_d_ln age), name(lws_age, replace)
	gen age_sq=age^2
	reg di_d_ln age age_sq

* (d) evaluating collinearity (VIFs)
	reg di_d_ln age age_sq
		vif
	replace age_sq=age_ct^2
 		reg di_d_ln age_ct age_sq
		vif

* (e) evaluating confounding
 	reg di_d_ln cistern		/* model without c_rec */
 	reg di_d_ln cistern	c_rec	/* model with c_rec */

* (f) interaction
	reg di_d_ln age_ct age_sq cistern##income_social ///
		c_rec m_prp_rural m_hdi
	reg di_d_ln age_ct age_sq cistern#income_social ///
		c_rec m_prp_rural m_hdi
		
* model building
	reg di_d_ln fam_n age_ct age_sq sex cistern income_social ///
		c_rec m_prp_rural m_hdi
		* drop fam_n sex
	reg di_d_ln age_ct age_sq cistern#income_social ///
		c_rec m_prp_rural m_hdi

********************************************
* Linear regression exercise # 3
********************************************
 
* open the revised "Brazil" data set 
	use brazil_lin_reg_exc.dta, clear
	gen age_ct=age-16.6
	gen age_sq=age_ct^2

* run model
	reg di_d_ln age_ct age_sq cistern#income_social ///
		c_rec m_prp_rural m_hdi

* (a) evaluate homscedasticity
	estat hettest
	rvfplot, name(hetero, replace)

* (b) evaluate normality
		predict rsta, rstandard
	* first graphically
		histogram rsta, start(-4.5) width(0.5) normal name(nrm_hist, replace)
		qnorm rsta, name(nrm_qq, replace)
	* then with a formal test
		swilk rsta
		drop rsta

* (c) examine detailed aspects of fit for the original model
	*generate necessary residuals and fit statistics
		capture drop yhat rsta rstu cooksd lev dfit
		predict yhat, xb
		predict rsta, rstandard
		predict rstu, rstudent
		predict cooksd, cooksd
		predict lev, leverage
		predict dfit, dfit
		predict db_hdi, dfbeta(m_hdi)

* compute threshold values
	scalar th_lev=2*(8+1)/496
	scalar th_cook=4/496
	scalar th_dfit=2*((8+1)/496)^0.5
	scalar th_db=2/(496^0.5)

	di "leverage threshold   " th_lev
	di "CooksD threshold   " th_cook
	di "dfits threshold   " th_dfit
	di "dfbeta threshold   " th_db

* (c-i)largest outlier(s)
	sum rsta, d
	sort rsta
	list di_d_ln age cistern income_social c_rec m_prp_rural m_hdi yhat rsta ///
		if abs(rsta)>3, clean noobs

* (c-ii) examine influential observations
	sort lev
	sum lev, d
		/* quite a few high leverage observations - high values of p_rct) */
	* list just the 5 highest leverage values
	list di_d_ln age cistern income_social c_rec m_prp_rural m_hdi ///
		yhat rsta lev in -5/-1 , clean noobs
		/*older people and, in one case, a very large community */
	sort cook
	list di_d_ln age yhat rsta lev cook dfit ///
		if (cook>th_cook | dfit>th_dfit | dfit<(-1*th_dfit) ) , clean noobs
		/* quite a few influential obs. - mixture of mod.-large residuals 
		and mod.-large levarge 
		- generally individuals with many days affected (di_d_ln>2) and either 
			quite young (<=3) or older (>=54) */

* (c-iii) examine dfbetas (for -m_hdi-)
	sort db_hdi
	list di_d_ln m_hdi yhat rsta lev cook db_hdi if ///
		(abs(db_hdi)>th_db) , clean noobs
		/* most influential obs. had large values of m_hdi (in top 10% of values ie >=0.69)
		- no big surprise here */
		
* refit the model three ways
* original model
	reg di_d_ln age_ct age_sq cistern#income_social ///
		c_rec m_prp_rural m_hdi
	estimates store orig
* first leave out |rsta| > 2.5 (n=4)
	reg di_d_ln age_ct age_sq cistern#income_social ///
		c_rec m_prp_rural m_hdi if abs(rsta)<2.8
	estimates store omit_rsta
* second leave out herds with high values of Cook's D (n=4)
	reg di_d_ln age_ct age_sq cistern#income_social ///
		c_rec m_prp_rural m_hdi if cooksd<0.02
	estimates store omit_cooksd
* third leave out herds with high values ofdbeta for -m_hdi- (n=4)
	reg di_d_ln age_ct age_sq cistern#income_social ///
		c_rec m_prp_rural m_hdi if abs(db_hdi)<0.16
	estimates store omit_db_hdi
* compare results
	estimates table orig omit*, stats(N) star
		* very little change in coefficients or sig. in any revised model
		* model appears to be quite robust
