* Chapter 15 - Model Building
* Sample problems
* mer15_smplprb.do
	version 12
	clear all
	set more off , permanent
	cd c:\mer\smplprb\mer15		/* this line is computer dependent */
	* capture log close 
	* log using mer15_smplprb, name(mer15_smplprb) text replace 

* creating the dataset to be analysed
	use "C:\mer\data\mi.dta", clear
		* transform -los- by adding 0.5 to the zero values and then log transofming
		gen los_ln=cond(los==0, log(los+0.5), log(los))
		* keep only the variables of interest
		keep id los_ln sex age white mar_c2 bmi prmi card cabg
		d
	save mi_los.dta, replace

* open the dataset
	use mi_los.dta, clear

* 1. causal diagram - no analyses required
	* causal sequence (sex white) age mar_c2 (bmi prmi) card cabg

* 2. identifying potential confounders - no analyses required
	
* 3. identifying intervening variables - no analyses required

* 4. identifying exposure independent variables - no analyses required

* 5. functional form of relationships between -age-, -bmi-, and -los_ln-
* (a) lowess curves
	lowess los_ln age, ti(lowess - age) name(lowess_age, replace) scheme(s1mono) msize(vsmall)
	lowess los_ln bmi, ti(lowess - bmi) name(lowess_bmi, replace)  scheme(s1mono) msize(vsmall)
	* some graphic evidence that both effects may be slightly non-linear
* (b) quadratic functions
	* center variables and create quadratic terms
	sum age bmi
	gen age_ct=age-72
	gen age_sq=age_ct^2
	gen bmi_ct=bmi-28
	gen bmi_sq=bmi_ct^2
	reg los_ln age_ct age_sq bmi_ct bmi_sq 
	* quadratic term for age is sig. (0.000) but bmi is borderline (0.070)
* (c) fractional polynomials
	fracpoly, compare: reg los_ln age sex white mar_c2 bmi prmi card cabg
		* borderline evidence (P=0.035) that a 2-term model (powers 3 3) 
		* better than a single term
	fracpoly, compare degree(1): reg los_ln age sex white mar_c2 bmi prmi card cabg
		* borderline evidence (P=0.041) that a transfromed single term 
		* (power -0.5) better than linear
		* conclusion - 2-term too complex, use transformed single term
	gen age_nsq=age^-0.5
	fracpoly, compare : reg los_ln bmi age_nsq sex white mar_c2 prmi card cabg
		* no evidence that 2 terms superior (P=0.128)
	fracpoly, compare degree(1): reg los_ln bmi age_nsq sex white mar_c2 prmi card cabg
		* minimal evidence (P=0.1) that transformed variable required
		* conclusion - leave -bmi- as linear

* 6. descriptive statistics, unconditional associations and pairwise correlations
	* (a) descriptive statistics
		sum los_ln bmi age_nsq sex white mar_c2 prmi card cabg
		* only 12% non-white and <7% had -card- or -cabg- .... may have limited power
	* (b) unconditional associations
		reg los_ln age_nsq	/* P=0.00 */
		reg los_ln bmi		/* P=0.73 */
		reg los_ln sex		/* P=0.00 */
		reg los_ln white	/* P=0.21 */
		reg los_ln mar_c2	/* P=0.00 */
		reg los_ln prmi		/* P=0.17 */
		reg los_ln card		/* P=0.14 */
		reg los_ln cabg		/* P=0.00 */
	* (c) pairwise correlations
		pwcorr bmi age_nsq sex white mar_c2 prmi card cabg
		* no pairwise correlations >0.31
	* (d) dropping variables - no additional analyses required
	
* 7. evaluation of interactions
	* interactions invovling -mar_c2-
	reg los_ln i.mar_c2##c.age_nsq sex white bmi prmi card cabg	/* P=0.014 */
	reg los_ln i.mar_c2##i.sex age_nsq white bmi prmi card cabg	/* P=0.041 */
	reg los_ln i.mar_c2##i.white age_nsq sex bmi prmi card cabg	/* P=0.700 */
	* Interactions involving -bmi-
	reg los_ln i.sex##c.bmi age_nsq white mar_c2 prmi card cabg	/* P=0.861 */
	reg los_ln i.white##c.bmi age_nsq sex mar_c2 prmi card cabg	/* P=0.012 */
	* interactions between -mar_c2- and -bmi-
	reg los_ln i.mar_c2##c.bmi age_nsq sex white prmi card cabg	/* P=0.080 */
	* no interactions significant at Bonferroni corrected P value (.05/6=0.008)
	
* 8. automated selection procedures
	* forward selection
	sw, pe(0.05) lockterm: reg los_ln (mar_c2 bmi) age_nsq sex white prmi card cabg
		estimates store fs
	* backward elimination
	sw, pr(0.05) lockterm: reg los_ln (mar_c2 bmi) age_nsq sex white prmi card cabg
		estimates store be
	* stepwise selection
	sw, pr(0.051) pe(0.05) lockterm: reg los_ln (mar_c2 bmi) age_nsq sex white prmi card cabg
		estimates store sw
	estimates table fs be sw, star(0.1 0.05 0.01)
	* no difference in models selected
	* sig. variables (in addition to -mar_c2- and -bmi-) are: -age_nsq-, -white-, -sex-, -cabg-
	
* 9. impact of confounders
	* full model
		reg los_ln age_nsq sex white mar_c2 bmi prmi card cabg
		estimates store full
	* drop -age_nsq-
		reg los_ln sex white mar_c2 bmi prmi card cabg
		estimates store dr_age
	* drop -sex-
		reg los_ln age_nsq white mar_c2 bmi prmi card cabg
		estimates store dr_sex
	* drop -white-
		reg los_ln sex age_nsq mar_c2 bmi prmi card cabg
		estimates store dr_white
	estimates table full dr_sex dr_white dr_age , star(0.1 0.05 0.01)
	
* 10. removal of intervening variables
	* full model
		reg los_ln age_nsq sex white mar_c2 bmi prmi card cabg
		estimates store full
	* model with intervening variables removed
		reg los_ln age_nsq sex white mar_c2 bmi prmi 
		estimates store no_int
	estimates table full no_int, star(.1 .05 .01)
	
* 11. final model and diagnostics
	reg los_ln age_nsq sex white mar_c2 bmi prmi 
	vif
		* no problem with collinearity
	predict rsta, rsta
	rvfplot, name(hetero, replace)
	estat hettest
		* graphic suggest increasing variance ... but test refutes this
	histogram rsta, normal name(normality1, replace)
	qnorm rsta , name(normality2, replace)
	swilk rsta
		* histogram suggest normality, but normal prob. plot and test suggest otherwise
	
* 12. evaluation of reliability
	gen rand=uniform()
		reg los_ln age_nsq sex white mar_c2 bmi prmi 
		estimates store all
	reg los_ln age_nsq sex white mar_c2 bmi prmi if rand<0.5
		estimates store half
		predict pv
	* compute the squared correlation between predicted values and observed
	* for the estimation subset and the validation subset
		display "R2 for half the data used to build the model:   " e(r2)
		corr pv los_ln if rand>=0.5
		display "R2 for 2nd half of data:   " r(rho)^2
		di (0.0401-0.0378)/0.0401
		* R2 dropped from 0.0401 to 0.0378 (6% drop)
		* conclusions - model has low predictive ability, but
		* model is reasonably robust
	* compare the coefficients
	reg los_ln age_nsq sex white mar_c2 bmi prmi if rand>=0.5
		estimates store otherhalf
		estimates table half otherhalf, star(.1 .05 .01)
	