* Chapter 18 - Modelling Count Data
* Sample problems
* mer18_smplprb.do
	version 13
	clear all
	set more off , permanent
	cd c:\mer\smplprb\mer18		/* this line is computer dependent */
	* capture log close 
	* log using mer18_smplprb, name(mer18_smplprb) text replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing sample problems for the reader. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/


*************************************************************************
* Creating the required dataset (-mi_cnt-)
*************************************************************************
	use "C:\mer\data\mi.dta", clear
	* evaluate the outcome of interest -los-
		histogram los, name(los, replace)
		sum los, detail
			* mean only slightly larger than median
	* checking linearity of -age-
		fracpoly, compare degree(1): poisson los age sex white mar_c2 bmi prmi card cabg
			* clear evidence (P=0.000) that a transfromed single term 
			* (power -1) better than linear
		fracpoly, compare degree(2): poisson los age sex white mar_c2 bmi prmi card cabg
			* conclusion - 2-term better, but for simplicty we will use transformed single term
		gen age_inv=(age^-1)
			* centre the variable (at avg. age of 67)
				sum age_inv
				replace age_inv=age_inv-0.015
				label var age_inv "transformed and centred age (1/age)-0.015"
	* centre -bmi-
		sum bmi, detail
		gen bmi_ct=bmi-28
			label var bmi_ct "centred bmi (bmi - 28)"
	* generate a variable that indicated if they died in hsopital or not
		gen died_hosp=(surv_mi==los)
			label var died_hosp "died in hospital"
	* keep only the variables of interest
		keep id los sex age age_inv white mar_c2 bmi bmi_ct prmi card cabg died_hosp
		order id los sex age age_inv white mar_c2 bmi bmi_ct prmi card cabg died_hosp
		d
* save the revised dataset
		save mi_cnt.dta, replace
	
****************************************************************************
* Sample problem questions
****************************************************************************

* 1. causal diagram - no analyses required
	* causal sequence (sex white) age mar_c2 (bmi prmi) card cabg
	
* 2. functional form of relationships between -age- and -los-
	use mi_cnt.dta, clear
* (a) quadratic function
	* center variables and create quadratic terms
	sum age
	gen age_ct=age-72
	gen age_sq=age_ct^2
	poisson los age_ct age_sq
	* quadratic term for age is sig. (0.000) 
	* a transformed version of -age- (-age_inv-) is provided in the data set -mi_cnt-
	drop age_ct age_sq

* 3. Poisson model
	use mi_cnt.dta, clear
* (a) Poisson model for # of days in hospital (intervening vars: -card- and -cabg- not included)
	poisson los sex age_inv white mar_c2 bmi_ct prmi
		estimates store pois_ml
		* only -sex- is not sig., but retained as a potential confounder
* (b) calculations of expected length of stay
		* baseline (female, age=67, non-white, not married, bmi=28, no prev. mi)
			display exp(_b[_cons])
		* baseline but married
			display exp(_b[_cons] + _b[mar])
		* longest stay individuals (age~=85 (age_inv=-0.003), bmi=48 (bmi_ct=20))
			* and had a prev. mi
			display exp(_b[_cons] - 0.003*_b[age_inv] + 20*_b[bmi_ct] +_b[prmi])		

* (c) fitting a Poisson - GLM framework
		glm los sex age_inv white mar_c2 bmi_ct prmi, fam(poisson) link(log)
			estimates store pois_glm
			* comparing ml and glm
				estimates table pois_ml pois_glm
* (d) expressing results as IRR
		glm los sex age_inv white mar_c2 bmi_ct prmi, fam(poisson) link(log)
		glm los sex age_inv white mar_c2 bmi_ct prmi, fam(poisson) link(log) eform
			codebook age_inv
* 4. Poisson model - diagnostics - overall model
	use mi_cnt.dta, clear
* (a) Comparing observed and predicted counts
	poisson los sex age_inv white mar_c2 bmi_ct prmi
	* using SPOST (-prcounts-) to compute predicted probabilities
	* Note: SPOST is a user written add-on to Stata
	* -prcounts- doesn't work after -glm- so I used the -poisson- command
		capture drop pc*
		prcounts pc, plot max(30) 
		twoway line pcpreq pcobeq pcval if pcval<=20, ///
			scheme(s1mono)lcolor(black black) name(obs_pred, replace) ///
			lpattern(solid dash) title("") legend(off) ///
			ytitle("proportion of observations", m(r+2)) ///
			xtitle("count", m(t+2)) plotregion(style(none)) ///
			text(.07 5 "observed", place(NE)) text(.05 8  "predicted", place(NE))
		capture drop pc*
* (b) Goodness of fit test
		poisson los sex age_inv white mar_c2 bmi_ct prmi
			estat gof
* (c) Scaled SE
		glm los sex age_inv white mar_c2 bmi_ct prmi, fam(poisson) link(log)
		glm los sex age_inv white mar_c2 bmi_ct prmi, fam(poisson) link(log) scale(x2)

* 5. Poisson model - diagnostics - residuals and Cook's distance
		glm los sex age_inv white mar_c2 bmi_ct prmi, fam(poisson) link(log)
* (a) residuals and Cook's d
		capture drop res_d res_p res_a cooks_d mu
		predict res_d, deviance standardized
		predict res_p, pearson  standardized
		predict res_a, anscombe standardized
		predict cooks_d, cooksd
		predict mu, mu
		* determine # of extreme residuals
			count if res_p>3	/* n=0  */
			count if res_p<-3	/* n=502 (19.1%) */
			* a lot of individuals with much longer stays than expected
			sort res_p
			* br id los died_hosp sex age_inv white mar_c2 bmi_ct prmi mu res_d res_p res_a cooks_d
	* list the smallest and largest residuals
		preserve
			keep if !missing(res_p)
			format bmi_ct mu res_p %4.2f
			format age_inv %6.4f
			format los died_hosp sex white mar_c2 prmi %4.0f
		sort res_p
			list id los died_hosp sex age white mar_c2 bmi prmi mu res_p in 1/3, clean noobs ab(5) 
			list id los died_hosp sex age white mar_c2 bmi prmi mu res_p in -10/-1, clean noobs ab(5)
		sort cooks_d
			list  id los died_hosp sex age white mar_c2 bmi prmi mu in -10/-1, clean noobs
			list  id res_d res_p res_a cooks_d in -10/-1, clean noobs
		restore
* (b) diagnostic plots
		twoway scatter res_a mu, name(r1, replace) ///
			scheme(s1mono) xtitle("Predicted count") msize(small) mcolor(black) ///
			text(20 6.5  "366", place(SE) size(small)) ///
			ytitle("Anscombe residuals", m(t+2)) plotregion(style(none)) 
		twoway scatter res_a cooks_d, name(r2, replace) ///
			scheme(s1mono) xtitle("Cook's distance") msize(small) mcolor(black) ///
			text(20 0.6  "366", place(SW) size(small)) ///
			ytitle("Anscombe residuals", m(t+2)) plotregion(style(none)) 	
		* combine the 2 residuals graphs
			graph combine r1 r2 , xsize(6) ysize(3) scheme(s1mono) ///
			subtitle("Diagnostic plots - Poisson regression")
* (c) refit the model without 366
		glm los sex age_inv white mar_c2 bmi_ct prmi, fam(poisson) link(log)
			estimates store p_all
		glm los sex age_inv white mar_c2 bmi_ct prmi  if id!=366, fam(poisson) link(log)
			estimates store p_all_1
		estimates table p_all p_all_1
			* approx. 27% reduction in coef. for mar_c2 (married)
				* estimated effect of mar_c2 quite sensitive to this observation
			* only a 3.1% reduction in coef. for bmi_ct
	
*************************************************************************
* Negative binomial regression
*************************************************************************
* open the dataset
	use mi_cnt.dta, clear
	* NB modelfor # of days in hospital (intervening vars: -card- and -cabg- not included)
	nbreg los sex age_inv white mar_c2 bmi_ct prmi
		estimates store nb_ml
		* note alpha is highly sig.
	* comparison of Poisson and NB models
		estimates table pois_ml nb_ml
			* fixed effects quite similar
		estimates stats pois_ml nb_ml
			* AIC indicates NB is far superior

*************************************************************************
* Ex 18.3 - 18.4 Comparing Negative binomial models fit using ML and GLM
*************************************************************************
	* comparing ML vs GLM
		glm los sex age_inv white mar_c2 bmi_ct prmi, fam(nb ml) link(log)
			/* note: -glm- does not estimate "alpha" by default so to get equivalent results
			it needs to be estimated using -nbreg- and then added to the -family- option.
			Alternatively, ml estimation may be specified and Stata estimates alpha before proceeding
			as shown above */
		estimates store nb_glm
		estimates table nb_ml nb_glm, b(%5.3f) se(%5.3f)
			* results identical (to 3 decimal places)
			
*************************************************************************
* Ex 18.5 Model diagnostics
*************************************************************************
	glm los sex age_inv white mar_c2 bmi_ct prmi, fam(nb ml) link(log)
	* deviance based goodness of fit 
		display chi2tail(e(df),e(deviance))
		* no evidence of lack of fit
	* Pearson goodness of fit test 
		display chi2tail(e(df),e(deviance_ps))	
		* lack of fit highly significant
	 
		capture drop res_d res_p res_a cooks_d mu
		predict res_d, deviance standardized
		predict res_p, pearson  standardized
		predict res_a, anscombe
		predict cooks_d, cooksd
		predict mu, mu
		sort cooks_d
		* determine # of extreme residuals
			count if res_p<-3 	/* n=0 */
			count if res_p>3		/* n=413 (15.7%) */
		* list the smallest and largest residuals and the largest Cook's Distances
		preserve
			keep if !missing(res_p)
			format bmi_ct mu res_p %4.2f
			format age_inv %6.4f
			format los died_hosp sex white mar_c2 prmi %4.0f
		sort res_p
			list id los died_hosp sex age white mar_c2 bmi prmi mu res_p in 1/3, clean noobs ab(5) 
			list id los died_hosp sex age white mar_c2 bmi prmi mu res_p in -10/-1, clean noobs ab(5)
		sort cooks_d
			list  id los died_hosp sex age white mar_c2 bmi prmi mu in -10/-1, clean noobs
			list  id res_d res_p res_a cooks_d in -10/-1, clean noobs
		restore

	* refit the model without 366
		glm los sex age_inv white mar_c2 bmi_ct prmi, fam(nb ml) link(log)
			estimates store nb_all
		glm los sex age_inv white mar_c2 bmi_ct prmi  if id!=366, fam(nb ml) link(log)
			estimates store nb_all_1
		estimates table p_all p_all_1 nb_all nb_all_1
	
*************************************************************************
* Ex 18.7 Zero inflated model
*************************************************************************
	tab los
	zinb los sex age_inv white mar_c2 bmi_ct prmi,  ///
		inflate(sex age_inv white mar_c2 bmi_ct prmi) nolog vuong
	estimates store zinfl
		* absolutely know evidence of zero inflation
		


